@@ -14,11 +14,11 @@
 #undef NDEBUG
 #include <assert.h>
 
-/* Must include main.c because it contains much more than just main() */
+// Must include main.c because it contains much more than just main()
 #define NO_VIM_MAIN
 #include "main.c"
 
-/* This file has to be included because the tested functions are static */
+// This file has to be included because the tested functions are static
 #include "charset.c"
 
 /*
@@ -38,7 +38,7 @@ test_isword_funcs_utf8(void)
     buf.b_p_isk = (char_u *)"@,48-57,_,128-167,224-235";
 
     curbuf = &buf;
-    mb_init(); /* calls init_chartab() */
+    mb_init(); // calls init_chartab()
 
     for (c = 0; c < 0x10000; ++c)
     {
@@ -76,6 +76,7 @@ test_isword_funcs_utf8(void)
     int
 main(void)
 {
+    estack_init();
     test_isword_funcs_utf8();
     return 0;
 }
