@@ -988,16 +988,6 @@ mch_settmode(int tmode)
 	mch_errmsg(_("cannot change console mode ?!\n"));
 }
 
-/*
- * set screen mode, always fails.
- */
-    int
-mch_screenmode(char_u *arg)
-{
-    emsg(_(e_screenmode));
-    return FAIL;
-}
-
 /*
  * Code for this routine came from the following :
  *
